use strict;
use warnings;

# this test was generated with Dist::Zilla::Plugin::Test::NoTabs 0.15

use Test::More 0.88;
use Test::NoTabs;

my @files = (
    'lib/Specio.pm',
    'lib/Specio/Coercion.pm',
    'lib/Specio/Constraint/AnyCan.pm',
    'lib/Specio/Constraint/AnyDoes.pm',
    'lib/Specio/Constraint/AnyIsa.pm',
    'lib/Specio/Constraint/Enum.pm',
    'lib/Specio/Constraint/Intersection.pm',
    'lib/Specio/Constraint/ObjectCan.pm',
    'lib/Specio/Constraint/ObjectDoes.pm',
    'lib/Specio/Constraint/ObjectIsa.pm',
    'lib/Specio/Constraint/Parameterizable.pm',
    'lib/Specio/Constraint/Parameterized.pm',
    'lib/Specio/Constraint/Role/CanType.pm',
    'lib/Specio/Constraint/Role/DoesType.pm',
    'lib/Specio/Constraint/Role/Interface.pm',
    'lib/Specio/Constraint/Role/IsaType.pm',
    'lib/Specio/Constraint/Simple.pm',
    'lib/Specio/Constraint/Structurable.pm',
    'lib/Specio/Constraint/Structured.pm',
    'lib/Specio/Constraint/Union.pm',
    'lib/Specio/Declare.pm',
    'lib/Specio/DeclaredAt.pm',
    'lib/Specio/Exception.pm',
    'lib/Specio/Exporter.pm',
    'lib/Specio/Helpers.pm',
    'lib/Specio/Library/Builtins.pm',
    'lib/Specio/Library/Numeric.pm',
    'lib/Specio/Library/Perl.pm',
    'lib/Specio/Library/String.pm',
    'lib/Specio/Library/Structured.pm',
    'lib/Specio/Library/Structured/Dict.pm',
    'lib/Specio/Library/Structured/Map.pm',
    'lib/Specio/Library/Structured/Tuple.pm',
    'lib/Specio/OO.pm',
    'lib/Specio/PP.pm',
    'lib/Specio/PartialDump.pm',
    'lib/Specio/Registry.pm',
    'lib/Specio/Role/Inlinable.pm',
    'lib/Specio/Subs.pm',
    'lib/Specio/TypeChecks.pm',
    'lib/Specio/XS.pm',
    'lib/Test/Specio.pm',
    't/00-report-prereqs.dd',
    't/00-report-prereqs.t',
    't/additional-exports.t',
    't/anon.t',
    't/any-does-isa.t',
    't/builtins-sanity.t',
    't/builtins.t',
    't/coercion.t',
    't/combines.t',
    't/conflicts.t',
    't/declare-helpers.t',
    't/dict.t',
    't/does-type.t',
    't/exception.t',
    't/import-twice.t',
    't/inheritance.t',
    't/inline-environment.t',
    't/inline.t',
    't/integer-edge-case.t',
    't/intersection.t',
    't/lib/Specio/Library/CannotSub.pm',
    't/lib/Specio/Library/Coercions.pm',
    't/lib/Specio/Library/Combines.pm',
    't/lib/Specio/Library/Conflict.pm',
    't/lib/Specio/Library/NoInline.pm',
    't/lib/Specio/Library/Union.pm',
    't/lib/Specio/Library/WithSubs.pm',
    't/lib/Specio/Library/XY.pm',
    't/library-with-subs.t',
    't/map.t',
    't/multiple-libraries.t',
    't/numeric-sanity.t',
    't/overloading-moose-bug.t',
    't/overloading.t',
    't/parameterized.t',
    't/perl-sanity.t',
    't/string-sanity.t',
    't/subs.t',
    't/t-clean.t',
    't/tuple.t',
    't/union-library.t',
    't/union.t',
    't/with-moo.t',
    't/with-moose.t'
);

notabs_ok($_) foreach @files;
done_testing;
