use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007006
use Test::Spelling 0.17;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
API
AnyCan
AnyDoes
AnyIsa
Builtins
CanType
Chris
ClassName
Coercion
Coercions
Constraint
DROLSKY
DROLSKY's
Dave
Declare
DeclaredAt
Dict
DoesType
Enum
Etheridge
Exception
Exporter
Graham
Helpers
Inlinable
Interface
Intersection
IsaType
Karen
Knop
Kogman
LaxVersionStr
Library
Lipatov
MUTC
Map
ModuleName
NegativeInt
NegativeNum
NegativeOrZeroInt
NegativeOrZeroNum
NonEmptySimpleStr
NonEmptyStr
Num
Numeric
OO
ObjectCan
ObjectDoes
ObjectIsa
PARAMETERIZABLE
PP
PackageName
Parameterizable
Parameterized
PartialDump
PayPal
Perl
PositiveInt
PositiveNum
PositiveOrZeroInt
PositiveOrZeroNum
RegexpRef
Registry
Role
Rolsky
Rolsky's
SIGNES
SPECIO
SafeIdentifier
ScalarRef
Simple
SimpleStr
SingleDigit
Specio
Str
StrictVersionStr
String
Structurable
Structured
Subs
Test
Throwable
Tuple
TypeChecks
Union
Vitaly
White
XS
Yuval
autarch
boolification
chrisw
coercions
cpansprout
de
distro
drolsky
ether
globification
haarg
inlinable
inline
isa
lav
lib
namespace
numification
parameterizable
parameterization
parameterized
reimplementation
sigils
slurpy
structurable
subtype
subtypes
